@@ -185,12 +185,30 @@ Launcher::Launcher(GarconMenuItem* item) :
 	{
 		m_search_command = normalize(command);
 	}
+
+	// Fetch desktop actions
+#ifdef GARCON_TYPE_MENU_ITEM_ACTION
+	GList* actions = garcon_menu_item_get_actions(m_item);
+	for (GList* i = actions; i != NULL; i = i->next)
+	{
+		GarconMenuItemAction* action = garcon_menu_item_get_action(m_item, reinterpret_cast<gchar*>(i->data));
+		if (action)
+		{
+			m_actions.push_back(new DesktopAction(action));
+		}
+	}
+	g_list_free(actions);
+#endif
 }
 
 //-----------------------------------------------------------------------------
 
 Launcher::~Launcher()
 {
+	for (std::vector<DesktopAction*>::size_type i = 0, end = m_actions.size(); i < end; ++i)
+	{
+		delete m_actions[i];
+	}
 }
 
 //-----------------------------------------------------------------------------
@@ -219,22 +237,18 @@ void Launcher::run(GdkScreen* screen) const
 			{
 			case 'i':
 				replace_with_quoted_string(command, i, "--icon ", garcon_menu_item_get_icon_name(m_item));
-				length = command.length() - 1;
 				break;
 
 			case 'c':
 				replace_with_quoted_string(command, i, garcon_menu_item_get_name(m_item));
-				length = command.length() - 1;
 				break;
 
 			case 'k':
 				replace_and_free_with_quoted_string(command, i, garcon_menu_item_get_uri(m_item));
-				length = command.length() - 1;
 				break;
 
 			case '%':
 				command.erase(i, 1);
-				length = command.length() - 1;
 				break;
 
 			case 'f':
@@ -247,9 +261,9 @@ void Launcher::run(GdkScreen* screen) const
 				// unsupported, pass in a list of URLs dropped on launcher
 			default:
 				command.erase(i, 2);
-				length = command.length() - 1;
 				break;
 			}
+			length = command.length() - 1;
 		}
 	}
 
@@ -278,6 +292,79 @@ void Launcher::run(GdkScreen* screen) const
 
 //-----------------------------------------------------------------------------
 
+void Launcher::run(GdkScreen* screen, DesktopAction* action) const
+{
+	const gchar* string = action->get_command();
+	if (exo_str_is_empty(string))
+	{
+		return;
+	}
+	std::string command(string);
+
+	// Expand the field codes
+	size_t length = command.length() - 1;
+	for (size_t i = 0; i < length; ++i)
+	{
+		if (G_UNLIKELY(command[i] == '%'))
+		{
+			switch (command[i + 1])
+			{
+			case 'i':
+				replace_with_quoted_string(command, i, "--icon ", action->get_icon());
+				break;
+
+			case 'c':
+				replace_with_quoted_string(command, i, action->get_name());
+				break;
+
+			case 'k':
+				replace_and_free_with_quoted_string(command, i, garcon_menu_item_get_uri(m_item));
+
+			case '%':
+				command.erase(i, 1);
+				break;
+
+			case 'f':
+				// unsupported, pass in a single file dropped on launcher
+			case 'F':
+				// unsupported, pass in a list of files dropped on launcher
+			case 'u':
+				// unsupported, pass in a single URL dropped on launcher
+			case 'U':
+				// unsupported, pass in a list of URLs dropped on launcher
+			default:
+				command.erase(i, 2);
+				break;
+			}
+			length = command.length() - 1;
+		}
+	}
+
+	// Parse and spawn command
+	gchar** argv;
+	gboolean result = false;
+	GError* error = NULL;
+	if (g_shell_parse_argv(command.c_str(), NULL, &argv, &error))
+	{
+		result = xfce_spawn_on_screen(screen,
+				garcon_menu_item_get_path(m_item),
+				argv, NULL, G_SPAWN_SEARCH_PATH,
+				garcon_menu_item_supports_startup_notification(m_item),
+				gtk_get_current_event_time(),
+				action->get_icon(),
+				&error);
+		g_strfreev(argv);
+	}
+
+	if (G_UNLIKELY(!result))
+	{
+		xfce_dialog_show_error(NULL, error, _("Failed to execute command \"%s\"."), string);
+		g_error_free(error);
+	}
+}
+
+//-----------------------------------------------------------------------------
+
 guint Launcher::search(const Query& query)
 {
 	// Prioritize matches in favorites and recent, then favories, and then recent
